from typing import Any
from pydantic import BaseModel
import json
from IPython.display import IFrame, display

class Dashboard(BaseModel):
    config: dict
    url: str

    def __init__(self, **kwargs):
        super().__init__(**kwargs)

        self.print_config()
        self.plot()

    def plot(self, width: int='100%', height: int=800):
        """plot the dashboard by remote url

        Args:
            width (int, optional): _description_. Defaults to 100%.
            height (int, optional): _description_. Defaults to 650.
        """
        display(
            IFrame(src=f"{self.url}", width=width, height=height)
        )

    def get_config(self) -> dict:
        """
        get the dashboard config
        """
        return self.config

    def print_config(self):
        """
        pretty print the cdashboard config
        """
        config = {k: v for k, v in self.config.items() if v is not None}

        print("Using config: ", end='')
        print(json.dumps(config, indent=4))
        
    def __print__(self) -> str:
        return f"Dashboard(config='{self.config}')"

    def __str__(self) -> str:
        return self.__print__()

    def __repr__(self) -> str:
        return self.__print__()


from typing import List, Optional, TypedDict, Dict


class ProjectConfig(TypedDict):
    project_type: str
    unique_identifier: str
    true_label: str
    tag: str
    pred_label: Optional[str]
    feature_exclude: Optional[List[str]]
    drop_duplicate_uid: Optional[bool]


class DataConfig(TypedDict):
    tags: List[str]
    feature_exclude: List[str]
    feature_encodings: Dict[str, str]
    drop_duplicate_uid: bool


class SyntheticDataConfig(TypedDict):
    model_name: str
    tags: List[str]
    feature_exclude: List[str]
    feature_include: List[str]
    feature_actual_used: List[str]
    drop_duplicate_uid: bool


class SyntheticModelHyperParams(TypedDict):
    # GPT2 hyper params
    batch_size: Optional[int]
    early_stopping_patience: Optional[int]
    early_stopping_threshold: Optional[float]
    epochs: Optional[int]
    model_type: Optional[str]
    random_state: Optional[int]
    tabular_config: Optional[str]
    train_size: Optional[float]

    # CTGAN hyper params
    epochs: Optional[int]
    test_ratio: Optional[float]



from aryaxai.client.client import APIClient
from aryaxai.common.xai_uris import GET_PROJECT_CONFIG, MODEL_SVG_URI
from typing import Dict, Optional
from pydantic import BaseModel, ConfigDict
import plotly.graph_objects as go
from IPython.display import SVG, display


class ModelSummary(BaseModel):
    project_name: str
    project_type: str
    unique_identifier: str
    true_label: str
    pred_label: Optional[str] = None
    metadata: Dict
    model_results: Dict
    is_automl: bool
    Source: str

    model_config = ConfigDict(protected_namespaces=())

    __api_client: APIClient

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        self.__api_client = kwargs.get("api_client")

    def info(self) -> dict:
        """Model Info

        :return: model info dict
        """
        info = {
            "source": self.Source,
            "model_name": self.model_results.get("model_name"),
            "model_type": self.model_results.get("model_type"),
            "model_param": self.model_results.get("model_params"),
            "data_tags_used_for_modelling": self.model_results.get("data_used_tags"),
            "modelling_info": self.model_results.get("modelling_info"),
        }

        return info

    def feature_importance(self):
        """Global features plot"""
        global_features = self.model_results.get("GFI")
        fig = go.Figure()

        fig.add_trace(
            go.Bar(
                y=list(global_features.keys()),
                x=list(global_features.values()),
                orientation="h",
            )
        )

        fig.update_layout(
            title="Global Feaure",
            xaxis_title="Values",
            yaxis_title="Features",
            width=800,
            height=600,
            yaxis_autorange="reversed",
            bargap=0.01,
            legend_orientation="h",
            legend_x=0.1,
            legend_y=1.1,
        )

        fig.show(config={"displaylogo": False})

    def prediction_path(self):
        """Prediction path plot"""
        model_name = self.model_results.get("model_name")
        res = self.__api_client.get(
            f"{MODEL_SVG_URI}?project_name={self.project_name}&model_name={model_name}"
        )

        if not res["success"]:
            raise Exception(res.get("details"))

        svg = SVG(res.get("details"))
        display(svg)

    def data_config(self):
        """returns data config for the project

        :return: response
        """
        model_name = self.model_results.get("model_name")
        res = self.__api_client.get(
            f"{GET_PROJECT_CONFIG}?project_name={self.project_name}&model_name={model_name}"
        )
        res["details"].pop("updated_by")
        res["details"]["metadata"].pop("path")
        res["details"]["metadata"].pop("avaialble_tags")

        return res.get("details")
